<?php
/*******************************************************************************
			File:					MapDraw.class.php
			Author:				Libor Maňák
*******************************************************************************/


require_once 'include/JSON.inc.php';
require_once 'class/PNGDraw.class.php';

class MapDraw
{
	var $map_width;
	var $map_height;
	var $zoom;

	var $png;
	
	// === MapDraw ===
	// $map_width 	- šířka mapy
	// $map_height  - výška mapy
	// $zoom    		- přiblížení mapy
	// Konstruktor třídy
	function MapDraw($map_width, $map_height, $zoom)
	{
		$this->zoom = $zoom / 100;
		$this->map_width = $map_width;
		$this->map_height = $map_height;

		$this->png = new PNGDraw($this->map_width, $this->map_height, $this->zoom);
	}

	// === render ==
	// Vyrenderuje mapu ve formátu PNG
	function render()
	{
		$this->png->render();
	}
	
	// === setBackgroundImage ===
	// $filename  - soubor
	// Vloží obrázek na pozadí
	function setBackgroundImage($filename)
	{
		$this->png->drawImage($filename, 0, 0);
	}
	
	// === loadMap ===
	// $jsondata
	// Načte mapu a připraví k vykreslení
	function loadMap($jsondata)
	{
		$data=json_decode($jsondata);

		foreach ($data as $key => $value) {
			if ($value->coords!="") {
				if (isset($value->coords)) {
					$value->coords = $this->processCoords($value->coords);
				}
				switch ($value->type) {
					case "line":
						$this->drawLine($value->coords, $value->color, $value->thickness);
						break;
					case "polyline":
						$this->drawPolyline($value->coords, $value->color, $value->thickness);
						break;
					case "polygon":
						$this->drawPolygon($value->coords, $value->color, $value->fillcolor, $value->thickness);
						break;
					case "rectangle":
						$this->drawRectangle($value->coords, $value->color, $value->fillcolor, $value->thickness);
						break;
					case "ellipse":
						$this->drawEllipse($value->coords, $value->color, $value->fillcolor, $value->thickness);
						break;
					case "circle":
						$this->drawCircle($value->coords, $value->color, $value->fillcolor, $value->thickness);
						break;
					case "arc":
						$this->drawArc($value->coords, $value->color, $value->fillcolor, $value->thickness);
						break;
					case "sign":
						$this->drawSign($value->coords, $value->id);
						break;
					case "fill":
						$this->fill($value->coords, $value->fillcolor);
						break;
 					case "text":
						//$this->drawText($value->coords, $value->color, $value->text);
						break;
					case "staircase":
						$this->drawStaircase($value->coords, $value->color, $value->thickness);
						break;
				}
			}
		}
	}
	
	// === drawRoomName ===
	// $coords - array(x1, x2, y1, y2)    1-min, 2-max
	// $name   - název/označení místnosti
	// Napíše název/označení místnosti
	function drawRoomName($coords, $name)
	{
	  $color = "6600FF";
	  $font = "arial.ttf";
	  $fontsize=9;
	  
		$x1=$coords[0];
		$x2=$coords[1];
		$y1=$coords[2];
		$y2=$coords[3];
		
		if ($this->zoom != 1) {
	    $x1 = $x1*$this->zoom;
	    $x2 = $x2*$this->zoom;
	    $y1 = $y1*$this->zoom;
	    $y2 = $y2*$this->zoom;
	    
	    $fontsize = round($fontsize*$this->zoom);
		}
		
		$xx = round( $x1 + (($x2-$x1)/2) );
		$yy = round( $y1 + (($y2-$y1)/2) );


	  $a = imagettfbbox($fontsize, 0, "fonts/$font", $name);
	  $text_width = $a[2] - $a[0];
	  $text_height = $a[1] - $a[7];
	  $text_width += 4;
	  
		if ($text_width > ($x2-$x1)) { //text se nevleze
		  if (($x2-$x1) < ($y2-$y1)) {
      	$angle = 90;
		  } else {
        $angle = 0;
			}
			
			$ok = false;
			while (!$ok) {
			  $a = imagettfbbox($fontsize, 0, "fonts/$font", $name);
			  $text_width = $a[2] - $a[0];
			  $text_height = $a[1] - $a[7];
			  $text_width += 4;
	  
				if ($angle == 90) {
	        if ($text_width > ($y2-$y1)) { //text se nevleze
	          $fontsize -= 1;
	        } else {
						$ok = true;
					}
				} else {
	        if ($text_width > ($x2-$x1)) { //text se nevleze
            $fontsize -= 1;
	        } else {
	          $ok = true;
					}
				}
				
				if ($fontsize<=5) {
					$ok = true;
				}
			}
		} else {
			$angle = 0;
		}
		
		
	  $this->drawText($name, $xx, $yy, $fontsize, $angle, $color, "fonts/$font", true);
	}

	function fillRoom($coords, $color)
	{
		$x1=$coords[0];
		$x2=$coords[1];
		$y1=$coords[2];
		$y2=$coords[3];

	/*	if ($this->zoom != 1) {
			$x1 = $x1*$this->zoom;
			$x2 = $x2*$this->zoom;
			$y1 = $y1*$this->zoom;
			$y2 = $y2*$this->zoom;
		}     */
		
		$xx = round( $x1 + (($x2-$x1)/2) );
		$yy = round( $y1 + (($y2-$y1)/2) );

		$this->png->fill($xx, $yy, $color);
	}


	function drawPathToRoom($coords)
	{
	  if ($coords!="") {
      $coords = $this->processCoords($coords);
      $this->drawStaircase(/*drawPolyline(*/$coords, "#FF0000", 5);
		}
	}
	
	
	///// ========== Private functions ========== /////
	
	
	// === processCoords ===
	// Rozparsuje koordináty
	private function processCoords($data)
	{
		$coords=explode(";", $data);

		for ($idx = 0; $idx < count($coords); ++$idx) {
			$coords[$idx]=explode(",", $coords[$idx]);
		}
		return $coords;
	}
	
	// === drawLine ===
	private function drawLine($coords, $color, $thickness)
	{
		for ($idx = 0; $idx < count($coords); $idx=$idx+2) {
			$this->png->drawLine($coords[$idx][0], $coords[$idx][1], $coords[$idx+1][0], $coords[$idx+1][1], $color, $thickness);
		}
	}

	// === drawPolyline ===
	private function drawPolyline($coords, $color, $thickness)
	{
		for ($idx = 1; $idx < count($coords); ++$idx) {
			$this->png->drawLine($coords[$idx-1][0], $coords[$idx-1][1], $coords[$idx][0], $coords[$idx][1], $color, $thickness);
		}
	}

	// === drawPolygon ===
	private function drawPolygon($coords, $color, $fillcolor, $thickness)
	{
		$points=array();
		for ($idx = 0; $idx < count($coords); ++$idx) {
			$points[]=$coords[$idx][0];
			$points[]=$coords[$idx][1];
		}
		$this->png->drawPolygon($points, $color, $fillcolor, $thickness);
	}

	// === drawRectangle ===
	private function drawRectangle($coords, $color, $fillcolor, $thickness)
	{
		for ($idx = 0; $idx < count($coords); ++$idx) {
			$this->png->drawRectangle($coords[$idx][0], $coords[$idx][1], $coords[$idx][2], $coords[$idx][3], $color, $fillcolor, $thickness);
		}
	}

	// === drawEllipse ===
	private function drawEllipse($coords, $color, $fillcolor, $thickness)
	{
		for ($idx = 0; $idx < count($coords); ++$idx) {
			$this->png->drawEllipse($coords[$idx][0], $coords[$idx][1], $coords[$idx][2], $coords[$idx][3], $color, $fillcolor, $thickness);
		}
	}

	// === drawCircle ===
	private function drawCircle($coords, $color, $fillcolor, $thickness)
	{
		for ($idx = 0; $idx < count($coords); ++$idx) {
			$this->png->drawCircle($coords[$idx][0], $coords[$idx][1], $coords[$idx][2], $color, $fillcolor, $thickness);
		}
	}

	// === drawArc ===
	private function drawArc($coords, $color, $fillcolor, $thickness)
	{
		for ($idx = 0; $idx < count($coords); ++$idx) {
			$this->png->drawArc($coords[$idx][0], $coords[$idx][1], $coords[$idx][2], $coords[$idx][3], $coords[$idx][4], $coords[$idx][5], $color, $fillcolor, $thickness);
		}
	}

	// === drawSign ===
	private function drawSign($coords, $id)
	{
		$url=dirname("http://".$_SERVER["SERVER_NAME"].$_SERVER["SCRIPT_NAME"]);
		if ($url[strlen($url)-1]!="/") {
			$url .= "/";
		}

		$this->png->drawImage($url."get_sign.php?id=$id", $coords[0][0],$coords[0][1]);
	}
	
	// === fill ===
	private function fill($coords, $fillcolor)
	{
		for ($idx = 0; $idx < count($coords); ++$idx) {
			$this->png->fill($coords[$idx][0], $coords[$idx][1], $fillcolor);
		}
	}

	// === drawStaircase ===
	private function drawStaircase($coords, $color, $thickness)
	{
		for ($idx = 1; $idx < count($coords); ++$idx) {
			$this->png->drawLine($coords[$idx-1][0], $coords[$idx-1][1], $coords[$idx][0], $coords[$idx][1], $color, $thickness);
		}

		$x1 = $coords[count($coords)-1][0];
		$y1 = $coords[count($coords)-1][1];
		$x2 = $coords[count($coords)-2][0];
		$y2 = $coords[count($coords)-2][1];
		
		$this->png->drawArrow(array($x2, $y2, $x1, $y1), $color, 10, 5);
	}

	// === drawText ===
	private function drawText($text, $x, $y, $size, $angle = 0, $color = "", $fontfile = "", $centerpoint = false)
	{
		$this->png->drawText($text, $x, $y, $size, $angle, $color, $fontfile, $centerpoint);
	}

}

?>