<?php
/*******************************************************************************
			File:					PNGDraw.class.php
			Author:				Libor Maňák

			Requirements:
		  	- PHP 5
		  	- GD 2.0.1 or later (2.0.28 or later is recommended)
*******************************************************************************/


class PNGDraw
{
	var $image;
	var $zoom;

	// === PNGDraw ===
	// $width 	- šířka obrázku
	// $height  - výška obrázku
	// $zoom    - přiblížení obrázku
	// Konstruktor třídy
	function PNGDraw($width, $height, $zoom)
	{
	  $this->zoom = $zoom;
		$this->image = imagecreatetruecolor($this->calcZoom($width), $this->calcZoom($height)) or die("Cannot Initialize new GD image stream");
	}

	// === render ==
	// Vyrenderuje PNG obrázek
	function render()
	{
		imagepng($this->image);
		imagedestroy($this->image);
	}

	// === drawLine ===
	function drawLine($x1, $y1, $x2, $y2, $color, $thickness = 1)
	{
		$this->pngline($x1, $y1, $x2, $y2, $this->colorAllocate($color), $thickness);
	}

	// === drawRectangle ===
	function drawRectangle($x, $y, $width, $height, $color = "", $fillcolor = "", $thickness = 1)
	{
		$this->pngrectangle($x, $y, $width, $height, $this->colorAllocate($color), $this->colorAllocate($fillcolor), $thickness);
	}

	// === drawEllipse ===
	function drawEllipse($cx, $cy, $width, $height, $color = "", $fillcolor = "", $thickness = 1)
	{
		$this->pngellipse($cx, $cy, $width, $height, $this->colorAllocate($color), $this->colorAllocate($fillcolor), $thickness);
	}

	// === drawCircle ===
	function drawCircle($cx, $cy, $average, $color = "", $fillcolor = "", $thickness = 1)
	{
		$this->pngellipse($cx, $cy, $average, $average, $this->colorAllocate($color), $this->colorAllocate($fillcolor), $thickness);
	}

	// === drawArc ===
	function drawArc($cx, $cy, $width, $height, $start, $end, $color = "", $fillcolor = "", $thickness = 1)
	{
		$this->pngarc($cx, $cy, $width, $height, $start, $end, $this->colorAllocate($color), $this->colorAllocate($fillcolor), $thickness);
	}

	// === drawPolygon ===
	function drawPolygon($points, $color = "", $fillcolor = "", $thickness = 1)
	{
		//$fillcolor="#FF00FF";
		$this->pngpolygon($points, $this->colorAllocate($color), $this->colorAllocate($fillcolor), $thickness);
	}

	// === drawImage ===
	function drawImage($filename, $x, $y)
	{
		$this->pngdrawimage($filename, $x, $y);
	}

	// === drawText ===
	function drawText($text, $x, $y, $size, $angle = 0, $color = "", $fontfile = "", $centerpoint = false)
	{
		$this->pngdrawtext($text, $x, $y, $size, $angle, $this->colorAllocate($color), $fontfile, $centerpoint);
	}
	
	// === fill ===
	function fill($x, $y, $fillcolor)
	{
		$this->pngfill($x, $y, $this->colorAllocate($fillcolor));
	}

	// === drawArrow ===
	function drawArrow($points, $fillcolor = "", $length = 10, $width = 5)
	{
		$this->pngarrow($points[0], $points[1], $points[2], $points[3], $length, $width, $this->colorAllocate($fillcolor));
	}
	

	///// ========== Private functions ========== /////


	// === calcZoom ==
	//  $value  - hodnota
	//  Přepočítá hodnotu na zoomovanou
	private function calcZoom($value)
	{
		return ($value * $this->zoom);
	}

	// === colorAllocate ===
	// $color  - barva, ve formátu: #FFFFFF
	//
	private function colorAllocate($color)
	{
		if ($color == "") {
			return -1;
		} else {
			$c=$this->html2rgb($color);
			return imagecolorallocate($this->image, $c[0], $c[1], $c[2]);
		}
		//imagecolorallocate, imagecolorallocatealpha, imagecolordeallocate
	}



	// === pngline ===
	// $x1, $y1, $x1, $y2	- souřadnice
	// $color  						-	identifikátor barvy
	// $thickness 				- šířka v pixelech
	private function pngline($x1, $y1, $x2, $y2, $color, $thickness = 1)
	{
		imagesetthickness($this->image, $thickness);
		imageline($this->image, $this->calcZoom($x1), $this->calcZoom($y1), $this->calcZoom($x2), $this->calcZoom($y2), $color);

		/*$thickness=$width-1;
		imagefilledpolygon($this->image, array($x1, $y1 - $width / 2, $x1, $y1 + $width / 2, $x2, $y2 + $width / 2, $x2, $y2 - $width / 2), 4, $color);*/
	}

	// === pngrectangle ===
	// $x, $y						- souřadnice
	// $width, $height  - šířka/výška
	// $color						- identifikátor okrajové barvy ("-1" - bezbarvy)
	// $fillcolor				-	identifikátor výplňové barvy ("-1" - bezbarvy)
	// $thickness 			- šířka v pixelech
	private function pngrectangle($x, $y, $width, $height, $color = -1, $fillcolor = -1, $thickness = 1)
	{
		imagesetthickness($this->image, $thickness);

		//výplň
		if ($fillcolor != -1) {
			imagefilledrectangle($this->image, $this->calcZoom($x), $this->calcZoom($y), $this->calcZoom($x+$width), $this->calcZoom($y+$height), $fillcolor);
		}

		//okraje
		if ($color != -1) {
			imagerectangle($this->image, $this->calcZoom($x), $this->calcZoom($y), $this->calcZoom($x+$width), $this->calcZoom($y+$height), $color);
		}
	}

	// === dpngellipse ===
	// $cx, $cy, $width, $height
	// $color			- identifikátor okrajové barvy ("-1" - bezbarvy)
	// $fillcolor	-	identifikátor výplňové barvy ("-1" - bezbarvy)
	// $thickness	- šířka okrajové čáry v pixelech
	private function pngellipse($cx, $cy, $width, $height, $color = -1, $fillcolor = -1, $thickness = 1)
	{
		imagesetthickness($this->image, $thickness);

		//výplň
		if ($fillcolor != -1) {
			imagefilledellipse($this->image, $this->calcZoom($cx), $this->calcZoom($cy), $this->calcZoom($width), $this->calcZoom($height), $fillcolor);
		}

		//okraje
		if ($color != -1) {
			imageellipse($this->image, $this->calcZoom($cx), $this->calcZoom($cy), $this->calcZoom($width), $this->calcZoom($height), $color);
		}
	}

	// === pngarc ===
	// $cx, $cy, $width, $height
	// $color			- identifikátor okrajové barvy ("-1" - bezbarvy)
	// $fillcolor  -	identifikátor výplňové barvy ("-1" - bezbarvy)
	// $thickness  - šíčka okrajové čáry v pixelech
	private function pngarc($cx, $cy, $width, $height, $start, $end, $color = -1, $fillcolor = -1, $thickness = 1)
	{
		imagesetthickness($this->image, $thickness);

		//výplň
		if ($fillcolor != -1) {
			imagefilledarc($this->image, $this->calcZoom($cx), $this->calcZoom($cy), $this->calcZoom($width), $this->calcZoom($height), $start, $end, $fillcolor, IMG_ARC_PIE);
		}

		//okraje
		if ($color != -1) {
			imagearc($this->image, $this->calcZoom($cx), $this->calcZoom($cy), $this->calcZoom($width), $this->calcZoom($height), $start, $end, $color);
		}
	}

	// === pngpolygon ===
	// $points
	// $color			- identifikátor okrajové barvy ("-1" - bezbarvy)
	// $fillcolor -	identifikátor výplňové barvy ("-1" - bezbarvy)
	// $thickness - šíčka okrajové čáry v pixelech
	private function pngpolygon($points, $color = -1, $fillcolor = -1, $thickness = 1)
	{
		imagesetthickness($this->image, $thickness);

		//nazoomovat body
		for ($idx = 0; $idx < count($points); ++$idx) {
			$points[$idx] = $this->calcZoom($points[$idx]);
		}

		//výplň
		if ($fillcolor != -1) {
			imagefilledpolygon($this->image, $points, count($points)/2, $fillcolor);
		}

		//okraje
		if ($color != -1) {
			imagepolygon($this->image, $points, count($points)/2, $color);
		}
	}


	// === pngdrawimage ===
	// $filename
	// $x, $y
	private function pngdrawimage($filename, $x, $y)
	{
		$insert_image = imagecreatefrompng($filename);
		list($img_width, $img_height, $img_type, $img_attr) = getimagesize($filename);

		if ($this->zoom==1) {
			imagecopy($this->image, $insert_image, $x, $y, 0, 0, $img_width, $img_height);
		} else {
			imagecopyresized($this->image, $insert_image, $this->calcZoom($x), $this->calcZoom($y), 0, 0, $this->calcZoom($img_width), $this->calcZoom($img_height), $img_width, $img_height);
		}
	}

	// === pngdrawtext ===
	// $text, $x, $y, $size, $angle, $color, $fontfile
	function pngdrawtext($text, $x, $y, $size, $angle = 0, $color = -1, $fontfile = "", $centerpoint = false)
	{
	  if ($centerpoint) {
	    $a = imagettfbbox($size, $angle, $fontfile, $text);
	    $width = $a[2] - $a[0];
	    $height = $a[1] - $a[/*7*/3];
	    if ($angle==90) {
      	imagettftext($this->image, $size, $angle, $x-round($width/2)+($size/2), $y+round($height/2), $color, $fontfile, $text);
      } else {
      	imagettftext($this->image, $size, $angle, $x-round($width/2), $y+round($height/2)+($size/2), $color, $fontfile, $text);
			}
		} else {
			imagettftext($this->image, $size, $angle, $x, $y, $color, $fontfile, $text);
		}
	}
	
	// === pngfill ===
	function pngfill($x, $y, $fillcolor = -1)
	{
		imagefill($this->image, $this->calcZoom($x), $this->calcZoom($y), $fillcolor);
	}

  // == pngarrow ===
	//Z nápovědy: http://php.net/manual/en/function.imageline.php
	function pngarrow($x1, $y1, $x2, $y2, $alength, $awidth, $color) {
		$distance = sqrt(pow($x1 - $x2, 2) + pow($y1 - $y2, 2));

		$dx = $x2 + ($x1 - $x2) * $alength / $distance;
		$dy = $y2 + ($y1 - $y2) * $alength / $distance;

		$k = $awidth / $alength;

		$x2o = $x2 - $dx;
		$y2o = $dy - $y2;

    $x3 = $y2o * $k + $dx;
    $y3 = $x2o * $k + $dy;

    $x4 = $dx - $y2o * $k;
    $y4 = $dy - $x2o * $k;

    $this->pngpolygon(array($x2, $y2, $x3, $y3, $x4, $y4), $color, $color, 1);
	}


	// == html2rgb ===
	// Staženo z: http://www.anyexample.com/programming/php/php_convert_rgb_from_to_html_hex_color.xml
	private function html2rgb($color)
	{
		if ($color[0] == '#')
			$color = substr($color, 1);

		if (strlen($color) == 6)
			list($r, $g, $b) = array($color[0].$color[1],
																$color[2].$color[3],
																$color[4].$color[5]);
		elseif (strlen($color) == 3)
			list($r, $g, $b) = array($color[0].$color[0], $color[1].$color[1], $color[2].$color[2]);
		else
			return false;

		$r = hexdec($r); $g = hexdec($g); $b = hexdec($b);

		return array($r, $g, $b);
	}

	// === rgb2html ===
	// Staženo z: http://www.anyexample.com/programming/php/php_convert_rgb_from_to_html_hex_color.xml
	private function rgb2html($r, $g=-1, $b=-1)
	{
		if (is_array($r) && sizeof($r) == 3)
			list($r, $g, $b) = $r;

		$r = intval($r); $g = intval($g);
		$b = intval($b);

		$r = dechex($r<0?0:($r>255?255:$r));
		$g = dechex($g<0?0:($g>255?255:$g));
		$b = dechex($b<0?0:($b>255?255:$b));

		$color = (strlen($r) < 2?'0':'').$r;
		$color .= (strlen($g) < 2?'0':'').$g;
		$color .= (strlen($b) < 2?'0':'').$b;
		return '#'.$color;
	}

}

?>